;
;*******************************************************
;*******************************************************
;**                                                   **
;**         Program prezentacji przerwan od czasu     **
;**                                                   **
;**         Andrzej Pawluczuk                         **
;**                                                   **
;**         Mikrokontroler: ATMEGA8515                **
;**                                                   **
;**             Bialystok, 2004                       **
;**                                                   **
;*******************************************************
;*******************************************************
;
	.nolist
;*******************************************************
.include "m8515def.inc"
;*******************************************************
	.list
	.listmac
;-----------------------------------------------------------------------------
.macro	ldz
	ldi	zl,low(@0)
	ldi	zh,high(@0)
.endm
;-----------------------------------------------------------------------------
.macro	pushz
	push	zl
	push	zh
.endm
;-----------------------------------------------------------------------------
.macro	popz
	pop	zh
	pop	zl
.endm
;-----------------------------------------------------------------------------
.macro	pushf
	push	acc
	in	acc,sreg
	push	acc
.endm
;-----------------------------------------------------------------------------
.macro	popf
	pop	acc
	out	sreg,acc
	pop	acc
.endm
;-----------------------------------------------------------------------------
.def acc		= r16
.def param		= r18
;-----------------------------------------------------------------------------
.equ	TimerData	= 0xA0
.equ	DivideConst	= 50
;-----------------------------------------------------------------------------
.equ	Stand_TIMSK	=	0b00000010
;	Timer/counter		  ||| | ||
;	Interrupt Mask		  ||| | |+-- OCIE0 : Timer/Counter0 Output Compare Match Interrupt Enable
;				  ||| | +--- TOIE0 : Timer/Counter 0 Overflow Interrupt Enable
;				  ||| +----- TICIE1 : Timer/Counter1 Input Capture Interrupt Enable
;				  ||+------- OCIE1B : Timer/counter1 Output Compare B Match Interrupt Enable
;				  |+-------- OCIE1A : Timer/counter1 Output Compare A Match Interrupt Enable
;				  +--------- TOIE1 : Timer/Counter1 Overflow Interrupt Enable
;-----------------------------------------------------------------------------
.equ	Stand_TCCR0	=	0b00000100
;	Timer/Counter0 Control	  ||||||||
;				  |||||||+-- CS00 : Clock Select
;				  ||||||+--- CS01 : Clock Select
;				  |||||+---- CS02 : Clock Select
;				  ||||+----- WGM01 : Waveform Generation Mode
;				  |||+------ COM00 : Compare Match Output Mode
;				  ||+------- COM01 : Compare Match Output Mode
;				  |+-------- WGM00 : Waveform Generation Mode
;				  +--------- FOC0 : Force Output Compare
;
;		CS02	CS01	CS00
;		0	0	0	timer/counter0 is stopped
;		0	0	1	clk
;		0	1	0	clk/8
;		0	1	1	clk/64
;		1	0	0	clk/256
;		1	0	1	clk/1024
;		1	1	0	external pin T0; falling edge
;		1	1	1	external pin T0; rising edge
;-----------------------------------------------------------------------------
.equ	Stand_PADir	=	0b11111111
;				  ||||||||
;				  |||||||+-- output :
;				  ||||||+--- output :
;				  |||||+---- output :
;				  ||||+----- output :
;				  |||+------ output :
;				  ||+------- output :
;				  |+-------- output :
;				  +--------- output :
.equ	Stand_PAData	=	0b11111111
;-----------------------------------------------------------------------------
.equ	Stand_PBDir	=	0b11111111
;				  ||||||||
;				  |||||||+-- output : LED
;				  ||||||+--- output : LED
;				  |||||+---- output : LED
;				  ||||+----- output : LED
;				  |||+------ output : LED
;				  ||+------- output : LED
;				  |+-------- output : LED
;				  +--------- output : LED
.equ	Stand_PBData	=	0b11111111
;-----------------------------------------------------------------------------
.equ	Stand_PCDir	=	0b11111111
;				  ||||||||
;				  |||||||+-- output :
;				  ||||||+--- output :
;				  |||||+---- output :
;				  ||||+----- output :
;				  |||+------ output :
;				  ||+------- output :
;				  |+-------- output :
;				  +--------- output :
.equ	Stand_PCData	=	0b11111111
;-----------------------------------------------------------------------------
.equ	Stand_PDDir	=	0b00000000
;				  ||||||||
;				  |||||||+-- input : KEY
;				  ||||||+--- input : KEY
;				  |||||+---- input : KEY
;				  ||||+----- input : KEY
;				  |||+------ input : KEY
;				  ||+------- input : KEY
;				  |+-------- input : KEY
;				  +--------- input : KEY
.equ	Stand_PDData	=	0b11111111
;-----------------------------------------------------------------------------
	.dseg
;*******************************************************
;*******************************************************
;**                                                   **
;**                  DATA  SEGMENT                    **
;**                                                   **
;*******************************************************
;*******************************************************
;						; VAR
TimeService :		.byte	2		;  TimeService : PROCEDURE ( ) ;
TimeDivider :		.byte	1		;  TimeDivider : BYTE ;
Status :		.byte	1		;  Status : BYTE ;
;-----------------------------------------------------------------------------
	.cseg
;*******************************************************
;*******************************************************
;**                                                   **
;**                  CODE  SEGMENT                    **
;**                                                   **
;*******************************************************
;*******************************************************
;-----------------------------------------------------------------------------
	.org	0				;
	rjmp	ResetProcessor			;
;-----------------------------------------------------------------------------
	.org	INT0addr			;External Interrupt0 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	INT1addr			;External Interrupt1 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	ICP1addr			;Input Capture1 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC1Aaddr			;Output Compare1A Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC1Baddr			;Output Compare1B Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OVF1addr			;Overflow1 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OVF0addr			;Overflow0 Interrupt Vector Address
	rjmp	TC0OvfInterrupt			;
;-----------------------------------------------------------------------------
	.org	SPIaddr				;SPI Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	URXCaddr			;USART Receive Complete Int. Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	UDREaddr			;USART Data Register Empty Int. Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	UTXCaddr			;USART Transmit Complete Int. Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	ACIaddr				;Analog Comparator Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	INT2addr			;External Interrupt2 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC0addr				;Output Compare0 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	ERDYaddr			;EEPROM Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	SPMaddr				;SPM complete Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
.include "util.inc"
;-----------------------------------------------------------------------------
HardwareInit :					;PROCEDURE HardwareInit ;
;*************					;BEGIN (* HardwareInit *)
	ldi	acc,Stand_TIMSK			; [TIMSK] := Stand_TIMSK ;
	out	TIMSK,acc			;
	ldi	acc,Stand_TCCR0			; [TCCR0] := Stand_TCCR0 ;
	out	TCCR0,acc			;
	ret					;END (* HardwareInit *) ;
;-----------------------------------------------------------------------------
EnvirInit :					;PROCEDURE EnvirInit ;
;**********					;BEGIN (* EnvirInit *)
	ldi	acc,Stand_PADir			; [DDRA] := Stand_PADir ;
	out	DDRA,Acc			;
	ldi	acc,Stand_PAData		; [PORTA] := Stand_PAData ;
	out	PORTA,acc			;
	ldi	acc,Stand_PBDir			; [DDRB] := Stand_PBDir ;
	out	DDRB,Acc			;
	ldi	acc,Stand_PBData		; [PORTB] := Stand_PBData ;
	out	PORTB,acc			;
	ldi	acc,Stand_PCDir			; [DDRC] := Stand_PCDir ;
	out	DDRC,Acc			;
	ldi	acc,Stand_PCData		; [PORTC] := Stand_PCData ;
	out	PORTC,acc			;
	ldi	acc,Stand_PDDir			; [DDRD] := Stand_PDDir ;
	out	DDRD,Acc			;
	ldi	acc,Stand_PDData		; [PORTD] := Stand_PDData ;
	out	PORTD,acc			;
	ret					;END (* EnvirInit *) ;
;-----------------------------------------------------------------------------
SoftwareInit :					;PROCEDURE SoftwareInit ;
;*************					;BEGIN (* SoftwareInit *)
	ldi	acc,0				; TimeDivider := 0 ;
	sts	TimeDivider,acc			;
	sts	Status,acc			; Status := 0 ;
	ldz	Play0Mode			; TimeService := Play0Mode ;
	sts	TimeService,zl			;
	sts	TimeService+1,zh		;
	ret					;END (* SoftwareInit *) ;
;-----------------------------------------------------------------------------
TC0OvfInterrupt :				;PROCEDURE TC0OvfInterrupt ;
;****************				;BEGIN (* TC0OvfInterrupt *)
	pushf					;
	pushz					;
	ldz	TimeDivider			; TimeDivider := TimeDivider + 1 ;
	ld	acc,Z				;
	inc	acc				;
	st	Z,acc				;
	cpi	acc,DivideConst			; IF TimeDivider = DivideConst THEN
	brne	TC0OV_0				; BEGIN
	ldi	acc,0				;  TimeDivider := 0 ;
	st	Z,acc				;
	ldz	TimeService			;  TimeService ( ) ;
	ld	acc,z+				;
	push	acc				;
	ld	acc,z				;
	mov	zh,acc				;
	pop	zl				;
	icall					;
TC0OV_0		:				; END (* IF *) ;
	ldi	acc,TimerData			; TCNT0 := TimerData ;
	out	TCNT0,acc			;
	popz					;
	popf					;
	reti					;END (* TC0OvfInterrupt *) ;
;-----------------------------------------------------------------------------
Play0Mode :					;PROCEDURE Play0Mode ( ) ;
;**********					;BEGIN
	ldz	Status				; Status := Status + 1 ;
	ld	acc,Z				;
	inc	acc				;
	st	Z,acc				;
	neg	acc				; [PORTB] := NOT Status ;
	out	PORTB,acc			;
	ret					;END (* Play0Mode *) ;
;-----------------------------------------------------------------------------
Play1Mode :					;PROCEDURE Play1Mode ( ) ;
;**********					;BEGIN
	ldz	Status				; IF Status = 0 THEN
	ld	acc,z				;
	cpi	acc,0				;
	brne	Pl1M_0				; BEGIN
	ldi	acc,1				;  Status := 1 ;
	rjmp	Pl1M_1				; END (* IF ... *) ;
Pl1M_0	:					; ELSE
						; BEGIN
	lsl	acc				;  Status := ( Status << 1 ) ;
Pl1M_1	:					; END (* IF ... ELSE *) ;
	st	Z,acc				;
	com	acc				; [PORTB] := NOT Status ;
	out	PORTB,acc			;
	ret					;END (* Play1Mode *) ;
;-----------------------------------------------------------------------------
Play2Mode :					;PROCEDURE Play2Mode ( ) ;
;**********					;BEGIN
	ldz	Status				; IF Status = 0 THEN
	ld	acc,z				;
	cpi	acc,0				;
	brne	Pl2M_0				; BEGIN
	ldi	acc,0x80			;  Status := 0x80 ;
	rjmp	Pl2M_1				; END (* IF ... *) ;
Pl2M_0	:					; ELSE
						; BEGIN
	lsr	acc				;  Status := ( Status >> 1 ) ;
Pl2M_1	:					; END (* IF ... ELSE *) ;
	st	Z,acc				;
	com	acc				; [PORTB] := NOT Status ;
	out	PORTB,acc			;
	ret					;END (* Play2Mode *) ;
;-----------------------------------------------------------------------------
Play3Mode :					;PROCEDURE Play3Mode ( ) ;
;**********					;BEGIN
	ldz	Status				; CASE Status OF
	ld	acc,z				;
	cpi	acc,0				; | 0 :
	brne	Pl3M_0				;
	ldi	acc,0b11100111			;  [PORTB] := 0x11100111 ;
	out	PORTB,acc			;
	rjmp	Pl3M_9				;
Pl3M_0	:					;
	cpi	acc,1				; | 1 :
	brne	Pl3M_1				;
	ldi	acc,0b11011011			;  [PORTB] := 0b11011011 ;
	out	PORTB,acc			;
	rjmp	Pl3M_9				;
Pl3M_1	:					;
	cpi	acc,2				; | 2 :
	brne	Pl3M_2				;
	ldi	acc,0b10111101			;  [PORTB] := 0b10111101 ;
	out	PORTB,acc			;
	rjmp	Pl3M_9				;
Pl3M_2	:					;
	cpi	acc,3				; | 3 :
	brne	Pl3M_3				;
	ldi	acc,0b01111110			;  [PORTB] := 0b01111110 ;
	out	PORTB,acc			;
;	rjmp	Pl3M_9				;
Pl3M_3	:					; ELSE
Pl3M_9	:					; END (* CASE *) ;
	ld	acc,z				; Status := ( Status + 1 ) MOD 4 ;
	inc	acc				;
	andi	acc,0x03			;
	st	z,acc				;
	ret					;END (* Play3Mode *) ;
;-----------------------------------------------------------------------------
Play4Mode :					;PROCEDURE Play4Mode ( ) ;
;**********					;BEGIN
	ldz	Status				; CASE Status OF
	ld	acc,z				;
	cpi	acc,0				; | 0 :
	brne	Pl4M_0				;
	ldi	acc,0b01111110			;  [PORTB] := 0b01111110 ;
	out	PORTB,acc			;
	rjmp	Pl4M_9				;
Pl4M_0	:					;
	cpi	acc,1				; | 1 :
	brne	Pl4M_1				;
	ldi	acc,0b10111101			;  [PORTB] := 0b10111101 ;
	out	PORTB,acc			;
	rjmp	Pl4M_9				;
Pl4M_1	:					;
	cpi	acc,2				; | 2 :
	brne	Pl4M_2				;
	ldi	acc,0b11011011			;  [PORTB] := 0b11011011 ;
	out	PORTB,acc			;
	rjmp	Pl4M_9				;
Pl4M_2	:					;
	cpi	acc,3				; | 3 :
	brne	Pl4M_3				;
	ldi	acc,0b11100111			;  [PORTB] := 0b11100111 ;
	out	PORTB,acc			;
;	rjmp	Pl4M_9				;
Pl4M_3	:					; ELSE
Pl4M_9	:					; END (* CASE *) ;
	ld	acc,z				; Status := ( Status + 1 ) MOD 4 ;
	inc	acc				;
	andi	acc,0x03			;
	st	z,acc				;
	ret					;END (* Play4Mode *) ;
;-----------------------------------------------------------------------------
Play5Mode :					;PROCEDURE Play5Mode ( ) ;
;**********					;BEGIN
	ldz	Status				; Status := Status XOR 0xFF ;
	ld	acc,z				;
	com	acc				;
	st	z,acc				;
	com	acc				; [PORTB] := Status ;
	out	PORTB,acc			;
	ret					;END (* Play5Mode *) ;
;-----------------------------------------------------------------------------
Play6Mode :					;PROCEDURE Play6Mode ( ) ;
;**********					;BEGIN
	ldz	Status				; Status := ( Status + 1 ) MOD 2 ;
	ld	acc,z				;
	inc	acc				;
	andi	acc,0x01			;
	st	z,acc				;
	cpi	acc,1				; IF Status = 1 THEN
	brne	Pl6M_0				; BEGIN
	ldi	acc,0x55			;  [PORTB] := 0x55 ;
	out	PORTB,acc			;
	ret					;  RETURN ;
Pl6M_0	:					; END (* IF *) ;
	ldi	acc,0xAA			; [PORTB] := 0xAA ;
	out	PORTB,acc			;
	ret					;END (* Play6Mode *) ;
;-----------------------------------------------------------------------------
Play7Mode_A :					;PROCEDURE Play7Mode_A ( ) ;
;************					;BEGIN
	ldz	Status				; IF Status = 0 THEN
	ld	acc,z				;
	cpi	acc,0				;
	brne	Pl7M_0				; BEGIN
	ldi	acc,1				;  Status := 1 ;
	rjmp	Pl7M_1				; END (* IF ... *) ;
Pl7M_0	:					; ELSE
						; BEGIN
	lsl	acc				;  Status := ( Status << 1 ) ;
Pl7M_1	:					; END (* IF ... ELSE *) ;
	st	Z,acc				;
	cpi	acc,0x80			; IF Status = 0x80 THEN
	brne	Pl7M_2				; BEGIN
	ldz	Play7Mode_B			;  TimeService := Play7Mode_B ;
	sts	TimeService,zl			;
	sts	TimeService+1,zh		;
Pl7M_2	:					; END (* IF *) ;
	com	acc				; [PORTB] := NOT Status ;
	out	PORTB,acc			;
	ret					;END (* Play7Mode_A *) ;
;-----------------------------------------------------------------------------
Play7Mode_B :					;PROCEDURE Play7Mode_B ( ) ;
;************					;BEGIN
	ldz	Status				; IF Status = 0 THEN
	ld	acc,z				;
	cpi	acc,0				;
	brne	Pl7M_0B				; BEGIN
	ldi	acc,0x80			;  Status := 0x80 ;
	rjmp	Pl7M_1B				; END (* IF ... *) ;
Pl7M_0B	:					; ELSE
						; BEGIN
	lsr	acc				;  Status := ( Status >> 1 ) ;
Pl7M_1B	:					; END (* IF ... ELSE *) ;
	st	Z,acc				;
	cpi	acc,0x01			; IF Status = 0x01 THEN
	brne	Pl7M_2B				; BEGIN
	ldz	Play7Mode_A			;  TimeService := Play7Mode_A ;
	sts	TimeService,zl			;
	sts	TimeService+1,zh		;
Pl7M_2B	:					; END (* IF *) ;
	com	acc				; [PORTB] := NOT Status ;
	out	PORTB,acc			;
	ret					;END (* Play7Mode_B *) ;
;-----------------------------------------------------------------------------
SetTimeService :				;PROCEDURE SetTimeService ( NewService [z] : PROCEDURE ( ) ) ;
;***************				;BEGIN
	cli					; DI ( ) ;
	sts	TimeService,zl			; TimeService := NewService ;
	sts	TimeService+1,zh		;
	sei					; EI ( ) ;
	ret					;END (* SetTimeService *) ;
;-----------------------------------------------------------------------------
SetService :					;PROCEDURE SetService ( Key [param] : BYTE ) ;
;***********					;BEGIN
	cpi	param,0xFE			; IF Key = 0xFE THEN
	brne	SeSe_0				; BEGIN
	clr	acc				;  Status := 0 ;
	sts	Status,acc			;
	ldz	Play0Mode			;  SetTimeService ( Play0Mode ) ;
	rcall	SetTimeService			;
	ldi	acc,0xFF			;  [PORTB] := 0xFF ;
	out	PORTB,acc			;
	ret					;  RETURN ;
SeSe_0	:					; END (* IF *) ;
	cpi	param,0xFD			; IF Key = 0xFD THEN
	brne	SeSe_1				; BEGIN
	clr	acc				;  Status := 0 ;
	sts	Status,acc			;
	ldz	Play1Mode			;  SetTimeService ( Play1Mode ) ;
	rcall	SetTimeService			;
	ldi	acc,0xFF			;  [PORTB] := 0xFF ;
	out	PORTB,acc			;
	ret					;  RETURN ;
SeSe_1	:					; END (* IF *) ;
	cpi	param,0xFB			; IF Key = 0xFB THEN
	brne	SeSe_2				; BEGIN
	clr	acc				;  Status := 0 ;
	sts	Status,acc			;
	ldz	Play2Mode			;  SetTimeService ( Play2Mode ) ;
	rcall	SetTimeService			;
	ldi	acc,0xFF			;  [PORTB] := 0xFF ;
	out	PORTB,acc			;
	ret					;  RETURN ;
SeSe_2	:					; END (* IF *) ;
	cpi	param,0xF7			; IF Key = 0xF7 THEN
	brne	SeSe_3				; BEGIN
	clr	acc				;  Status := 0 ;
	sts	Status,acc			;
	ldz	Play3Mode			;  SetTimeService ( Play3Mode ) ;
	rcall	SetTimeService			;
	ldi	acc,0xFF			;  [PORTB] := 0xFF ;
	out	PORTB,acc			;
	ret					;  RETURN ;
SeSe_3	:					; END (* IF *) ;
	cpi	param,0xEF			; IF Key = 0xEF THEN
	brne	SeSe_4				; BEGIN
	clr	acc				;  Status := 0 ;
	sts	Status,acc			;
	ldz	Play4Mode			;  SetTimeService ( Play4Mode ) ;
	rcall	SetTimeService			;
	ldi	acc,0xFF			;  [PORTB] := 0xFF ;
	out	PORTB,acc			;
	ret					;  RETURN ;
SeSe_4	:					; END (* IF *) ;
	cpi	param,0xDF			; IF Key = 0xDF THEN
	brne	SeSe_5				; BEGIN
	ldi	acc,0x0F			;  Status := 0x0F ;
	sts	Status,acc			;
	ldz	Play5Mode			;  SetTimeService ( Play5Mode ) ;
	rcall	SetTimeService			;
	ldi	acc,0xFF			;  [PORTB] := 0xFF ;
	out	PORTB,acc			;
	ret					;  RETURN ;
SeSe_5	:					; END (* IF *) ;
	cpi	param,0xBF			; IF Key = 0xBF THEN
	brne	SeSe_6				; BEGIN
	clr	acc				;  Status := 0 ;
	sts	Status,acc			;
	ldz	Play6Mode			;  SetTimeService ( Play6Mode ) ;
	rcall	SetTimeService			;
	ldi	acc,0xFF			;  [PORTB] := 0xFF ;
	out	PORTB,acc			;
	ret					;  RETURN ;
SeSe_6	:					; END (* IF *) ;
	cpi	param,0x7F			; IF Key = 0x7F THEN
	brne	SeSe_7				; BEGIN
	clr	acc				;  Status := 0 ;
	sts	Status,acc			;
	ldz	Play7Mode_A			;  SetTimeService ( Play7Mode_A ) ;
	rcall	SetTimeService			;
	ldi	acc,0xFF			;  [PORTB] := 0xFF ;
	out	PORTB,acc			;
	ret					;  RETURN ;
SeSe_7	:					; END (* IF *) ;
	ret					;END (* SetService *) ;
;-----------------------------------------------------------------------------
ProgDelay :					;PROCEDURE ProgDelay ( ) ;
;**********					;BEGIN
	ldz	0				;
PrDe_0	:					;
	adiw	zl,1				;
	mov	acc,zl				;
	or	acc,zh				;
	brne	PrDe_0				;
	ret					;END (* ProgDelay *) ;
;-----------------------------------------------------------------------------
ResetProcessor :				;
	cli					;
	ldi	acc,HIGH(RAMEND)		;
	out	SPH,acc				;
	ldi	acc,LOW(RAMEND)			;
	out	SPL,acc				;
						;BEGIN (* Main *)
	rcall	ClearRAM			; ClearRAM ( ) ;
	rcall	ClearRegs			; ClearRegs ( ) ;
	rcall	HardwareInit			; HardwareInit ( ) ;
	rcall	EnvirInit			; EnvirInit ( ) ;
	rcall	SoftwareInit			; SoftwareInit ( ) ;
	sei					; EI ( ) ;
Main_0	:					; LOOP
	in	acc,PIND			;  Key := PIND ( ) ;
	cpi	acc,0xFF			;  IF Key <> 0xFF THEN
	breq	Main_1				;  BEGIN
	rcall	ProgDelay			;   ProgDelay ( ) ;
	in	acc,PIND			;   Key := PIND ( ) ;
	cpi	acc,0xFF			;   IF Key <> 0xFF THEN
	breq	Main_2				;   BEGIN
	mov	param,acc			;    SetService ( Key ) ;
	rcall	SetService			;
Main_4	:					;    WHILE PIND ( ) <> 0xFF DO
	in	acc,PIND			;
	cpi	acc,0xFF			;
	breq	Main_3				;    BEGIN
	rjmp	Main_4				;
Main_3	:					;    END (* WHILE *) ;
	rcall	ProgDelay			;    ProgDelay ( ) ;
Main_2	:					;   END (* IF *) ;
Main_1	:					;  END (* IF *) ;
	rjmp	Main_0				; END (* LOOP *) ;
						;END (* Main *) ;
;-----------------------------------------------------------------------------
.exit
